#ifndef GATE
#define GATE
#include "UpdateListener.h"
#include "InSignal.h"
#include "OutSignal.h"
#include "Wire.h"
#include "Simulator.h"
#include <string>
using namespace std;
/* This class encapsulates common gates features and behavoirs.
   Any gate has one output connected to a wire.
   Any gate has a delay.
   Each gate has a reference to a Simulator object to
   register future events. 
   The logic of each gate must be defined in a
   specific inherited class */ 

class Gate:public UpdateListener, public InSignal,public OutSignal{
  private:
    const int id;  // id for a gate within its class
    float delay;
    bool futureOutput, output;
    vector<InSignal*> drains;
    Simulator* p_simulator;
  protected:
    Gate (float d, int nextId, Simulator* ps);
    virtual string getSimpleName()const =0;
  public:
    virtual bool computeOutput()=0;
    virtual void updateOutputValue(bool newValue) { output = newValue;} // UpdateListener class
    virtual void propagateOutputValue();    // UpdateListener
    virtual void inputValueChanged();  // InSignal class
    virtual void connectInput(OutSignal *p_output)=0;  // InSignal class
    virtual void addSource(OutSignal * p_output)=0;     // InSignal class
    virtual bool getValue() const { return output; }       // OutSignal class
    virtual void connectOutput(InSignal* p_drain);     // OutSignal class 
    virtual void addDrain(InSignal * p_drain) {drains.push_back(p_drain);}
    virtual string getDescription() const;  //  OutSignal class
}; 
#endif